function A = activation(p,smap,Nd)

% ACTIVATION Obtain activation maps
%   
%       A = activation(p,smap,Nd)
%
% INPUTS
%      p: Data points in the input space
%   smap: UnioviSOM SOM structure
%     Nd: Neighborhood distance for lateral interaction (default=1)
%
% OUTPUTS:
%      A: (1xS) Vector containing the activations for each neuron
%
% DESCRIPTION
%      This function computes the resulting activation value for each neuron 
%      in the SOM by summing up all the independent activation patterns 
%      originated by each individual stimulus (input data point) in p. 
%      Gaussian lateral interaction with width Nd is considered.
%
% EXAMPLE
%      p = [randn(3,100)+3 randn(3,100)-3]; % generate two clouds of 3D points
%      t = [randn(3,500)+3];                % test set similar to the first cloud
%      som = inisom(p,{30,30},{'x','y','z'});             % initialize a SOM (PCA initialization)
%      som = bsom(p,som,10,1);              % train a SOM (10 epochs, final neigborhood=1)
%      a   = activation(p(:,1:100),som,2);    % compute the activation of the first cloud
%      figure(1); 
%      planes(som);                         % plot the SOM planes
%      figure(2);
%      planes(som.pos,a);                   % plot the activation map of the test set
%      title('activation map');


%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


if nargin==2
   Nd = 1;
end


[qerr,bmu] = min(matdist(smap.w',p)); 
hci = gaussfun(double(smap.dis(:,bmu)),[Nd 0]);
hij = gaussfun(double(smap.dis),[Nd 0]);
A   = sum(hci,2)./sum(hij,2);
A   = A';


    
