function [PCi, PxCj] = bayesclass(x,C,sigma)
% BAYESCLASS Simple bayes classification method
%
% CALLS
%   [PCi, PxCj] = bayesclass(x,C)
%   [PCi, PxCj] = bayesclass(x,C,sigma)
% 
% INPUTS
%     x: Data matrix of size [R,Q]
%     C: Cell array with P data matrices of size [R,Qi]. Each data matrix contains points 
%        of a given class
% sigma: Scalar width factor (optional)
%
% OUTPUTS
%   PCi: Matrix [P,Q]. The i-th row contains the probability of
%        class Ci for each column of x
%  PxCj: Matrix [P,Q] with P(x|Cj)
%
% DESCRIPTION
%        Performs a naive Bayes classification of points (columns) of data
%        matrix x, using a Parzen kernel estimation of densities for different
%        classes.


%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



Q = size(x,2);
NumClass = length(C);



% Tipically there are few classes so a FOR 
% loop is not very expensive
Nt = 0;
for j = 1:NumClass
    N(j) = size(C{j},2);
    dis = sort(matdist(C{j},C{j}));
    if nargin==2
        sigma{j} = mean(sqrt(dis(2:4,:)));
    end    
end

% BAYES RULE FOR P(Ci)
% P(Cj) = P(x|Cj)*P(Cj)/(P(x|C1)*P(C1)+P(x|C2)*P(C2)+...P(x|Cn)*P(Cn))


Prior = N/sum(N);                                         % P(Cj)
for j = 1:NumClass
    PxCj(j,:) = sum(mvg(x,C{j},sigma{j})')/N(j);          % P(x|Cj)
end

PCi = PxCj.*(Prior'*ones(1,Q))./(ones(NumClass,1)*Prior*PxCj);
    
