function [smap,MSE] = bibmatsom(p,smap,epochs,Nc,weights,Bidx)
% BIBMATSOM Train a SOM using the batch SOM training algorithm (memory-efficient version)
%
% CALLS
%       [som,mse] = bsom(p)
%       [som,mse] = bsom(p,smap)
%       [som,mse] = bsom(p,smap,epochs,Nc)
%       [som,mse] = bsom(p,smap,epochs,Nc,weights)
%       [som,mse] = bsom(p,smap,epochs,Nc,weights,Bidx)
%
% INPUTS
%       p: Input data matrix (R x Q) where each column correspond to each sample vector
%          in the input space.
%    smap: SOM structure (see INISOM)
%  epochs: Number of times (iterations) which the whole input data set is used for 
%          training
%      Nc: Neighbourhood. It can be either one of these:
%            - a single scalar specifying the last value of a monotonically decreasing
%              neighbourhood.
%            - a (1 x 2) vector with the starting and finishing value of the neighbourhood
%            - a (1 x epochs) vector with values of the neighbourhood at each epoch
% weights: Weight values given to all the samples (by default all samples have the same 
%          weight)
% Bidx:    Index vector containing variables to be taken into account in the 
%          self-organization process
% 
% 
% EXAMPLE
% Trains a SOM ordered by variables 2 and 3
% 
% p = rands(3,500); 
% p = [p(1:3,:); [(1-p(1,:).^2)].*[(1-p(2,:).^2)].*[tansig(p(3,:))]; randn(2,500)];
% som = inisom(p,{30,30});
% som = bibmatsom(p,som,10,1,ones(1,size(p,2)),[2,3]); 
% planes(som)
%
%
%
% OUTPUTS
%     som: Self-Organizing Map structure
%     mse: Vector with the mean squared quantization error for each epoch.
%
% DESCRIPTION
%   Trains the SOM parmeters specified in the smap structure using the so-called batch SOM
%   algorithm with lower memory usage (but higher CPU time) than the BSOM function.
%
%   By default uses a monotonically (logaritmic) decreasing neighbourhood. 
%
%   It also allows to specify weight values to account for the contribution of each 
%   sample to the density function in the input space.
%
% See also INISOM, PLANES, PLANES3D, SOMINTERP, FPROJ, BPROJ

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


if nargin == 1
    smap = inisom(p);
end

if nargin <= 2
    epochs = 50;
    Nc = 1;
end


Q   = size(p,2);

% Neighbourhood selection
lnc = length(Nc);
if lnc==1
    neigh = logspace(log10(max([cat(1,smap.dims{:})]/3)),log10(Nc),epochs);
elseif lnc==2
    neigh = logspace(log10(Nc(1)),log10(Nc(2)),epochs);
elseif lnc>2
    if lnc~=epochs
        error('length of neighborhood vector does not match number of epochs');
    end;
    neigh = Nc;
end;



% Batch SOM algorithm
if nargin<6,
    Bidx = 1:smap.R;
end
ELEM_PASO=1000;
for ke = 1:epochs
    pasos=fix(Q/ELEM_PASO);
    restopasos=mod(Q,ELEM_PASO);
    num=zeros(smap.S,smap.R);
    den=zeros(smap.S,1);
    qerr=0;
    for kp=0:pasos-1
        pk=p(:,(kp*ELEM_PASO+1):(kp+1)*ELEM_PASO);
        [m,i] = min(matdist(smap.w(:,Bidx)',pk(Bidx,:)),[],1); 
        hci   = gaussfun(double(smap.dis(:,i)),[neigh(ke),0]);
        
        if nargin==5
            hci   = hci.*(ones(smap.S,1)*weights);
        end
    
        num=num+hci*pk';
        den=den+sum(hci,2);
        qerr=qerr+sum(m);
        fprintf('.');
    end    
    if(restopasos)
        pk=p(:,pasos*ELEM_PASO+1:end);
        [m,i] = min(matdist(smap.w(:,Bidx)',pk(Bidx,:)),[],1); 
        hci   = gaussfun(double(smap.dis(:,i)),[neigh(ke),0]);
        
        if nargin==5
            hci   = hci.*(ones(smap.S,1)*weights);
        end
    
        num=num+hci*pk';
        den=den+sum(hci,2);
        qerr=qerr+sum(m);
        fprintf('.\n');
    end
    
    smap.w=num./(den*ones(1,smap.R));
    
    %MSE(ke) = mean(m);
    MSE(ke)=qerr/Q;
    if ~rem(ke,epochs/10)
        fprintf('\nEpoca: %d,\t Vecindad: %f,\t mse: %f\t mse (p.u.): %f \n',ke,neigh(ke),MSE(ke), MSE(ke)/MSE(1));
    end
    
    
end


