function [pr,res] = bproj(g,smap,projtype)

% BPROJ Backward Projection using the SOM
%
% [pr,res,bmu] = bproj(g,smap,projtype)
%
%        g: (Rout x Q) vector in the output space with data points by columns
%     smap: SOM struct (see INISOM, BSOM)
% projtype: String with the projection type:
%           'discrete: Original discrete SOM projection (default)
%           'grnn':    Generalized Regression Neural Network
%           'ls':      Uses seast squares (Ridge Regression) to compute 
%                      the weight matrix
%
% CALLS
%           [pr,res,bmu] = bproj(p,smap)
%           [pr,res,bmu] = bproj(p,smap,projtype)
%
% DESCRIPTION
%    Projects output space samples (given as columns of data matrix g) onto
%    the input space using the projection defined by the som 'smap'. Projection 
%    can be either discrete or continuous.
%
% See also FPROJ

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


if nargin==3
    % Continuous SOM projection
    if strncmp(projtype,'discrete',1)
        % Discrete SOM projection
        [qerr,bmu] = min(matdist(smap.pos,g));
        gest = smap.pos(:,bmu);
        pr = smap.w(bmu,:)';        
    elseif strncmp(projtype,'grnn',1)
        grnnbwd = grnntra(smap.w',smap.pos);    % Backward (grid to input) projection GRNN
        grnnslf = grnntra(smap.pos,smap.pos);   % Self     (grid to grid)  projection GRNN
        pr   = grnnsim(g,grnnbwd);
        gest = grnnsim(g,grnnslf);
    elseif strncmp(projtype,'ls',1)
        grnnbwd = grnntra(smap.w',smap.pos,0.0001,'e');    % Backward (grid to input) projection GRNN
        grnnslf = grnntra(smap.pos,smap.pos,0.0001,'e');   % Self     (grid to grid)  projection GRNN
        pr   = grnnsim(g,grnnbwd);
        gest = grnnsim(g,grnnslf);
    end
else
    % Discrete SOM projection
    [qerr,bmu] = min(matdist(smap.pos,g));
    gest = smap.pos(:,bmu);
    pr = smap.w(bmu,:)';
end

res = g - gest;