function [p,mu,C] = clouddata(mu,sigma,Q)
% CLOUDDATA Generates clouds of points for testing algorithms
%
%        [p,mu,C] = clouddata(mu,sigma,Q)
% 
%
%    mu: Matrix (RxNc) with the clouds centers by columns
% sigma: Defines the covariance matrix of each cloud. Can be
%        either:
%        'full':           Uses random full covariances
%        scalar number:    Uses spherical covs sigma*I
%        vector (1xNc):    Uses spherical covs of 
%                          different widths, sigma(k)*I
%        n-array (R,R,Nc): Assigns a specified cov. matrix
%                          to each cloud
%     Q: Number of points in each cloud (optional. default=100). 
%        Can be either:
%           a scalar: with the number of points for all clouds
%           a vector: (1xNc) with the number of points of each cloud
%
% DESCRIPTION
%   Generates a data set composed of multivariate gaussian 
%   distributed clouds of data allowing for specification of
%   spherical or ellipsoidal covariances


%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


R  = size(mu,1);
Nc = size(mu,2);

if nargin==1
    for k = 1:Nc
        sigma(:,:,k) = eye(R);
    end
elseif isnumeric(sigma)
    if length(sigma) == 1
        for k = 1:Nc
            C(:,:,k) = eye(R)*sigma;
        end
    elseif size(sigma,1) == 1
        for k = 1:Nc
            C(:,:,k) = eye(R)*sigma(k);
        end
    else
        C = sigma;
    end
else
    if strncmp(sigma,'full',1)
        for k = 1:Nc
            T = randn(R,R);
            C(:,:,k) = T'*T + 0.2*eye(R);
        end
    end
end

if nargin <= 2
    Q = 100*ones(1,Nc);
elseif length(Q)==1
    Q = Q*ones(1,Nc);
end
    

p = [];
for k = 1:Nc
    T = sqrtm(C(:,:,k));
    p = [p T*randn(R,Q(k))+mu(:,k)*ones(1,Q(k))];
end



     
    
    
    
    
    
    
    
