function [C,idxc,idxf,namesc,namesf] = correlationmatrix(C,names,zoom,sortopts)
% CORRELATIONMATRIX Interactive plot of correlation matrix
%
% CALLS
%       [C,idxc,idxf,namesc,namesf]=correlationmatrix(C,names)
%       [C,idxc,idxf,namesc,namesf]=correlationmatrix(C,names,zoom)
%       [C,idxc,idxf,namesc,namesf]=correlationmatrix(C,names,zoom,sortopts)
%
%        C: Matrix to be sorted
%    names: Can be either: 
%             - Empty string (to use default names)
%             - Cell array with strings (to be used both in rows and columns)
%             - Cell array containing two cell arrays of strings: one for
%             rows and other for columns
%     zoom: scalar number. 1=zoom mode, 0=no-zoom mode
% sortopts: Can be either:
%               ''   sort rows and columns by columns
%               'c'  sort columns by columns 
%               'f'  sort rows by rows 
%               'cf' sort rows by rows and columns by columns
%               'no' no sort
%
% DESCRIPTION
%       This tool allows to visualize big correlation matrices along with
%       the variable names, allowing to zoom in and out parts of the matrix
%       in an interactive way.

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


[R1,R2]=size(C);
if nargin<=3
    if R1~=R2
        disp('Obtaining correlation coefficients...');
        C = corrcoef(C');
        [R1,R2]=size(C);
    end
end

if length(names)==0         % if names is empty...
    for k = 1:R1
        namesf{k} = sprintf('x_{%d}',k);
    end
    for k = 1:R2
        namesc{k} = sprintf('x_{%d}',k);
    end
elseif (length(names)==2) && (iscell(names{1}))   % if names has independent labels for rows and columns...
        namesf = names{1};
        namesc = names{2};
else                        % if names contains only one set of names...
    if length(names)==R2
        namesc = names;
    else
        for k = 1:R2
            namesc{k} = sprintf('x_{%d}',k);
        end
    end
    if length(names)==R1
        namesf = names;
    else
        for k = 1:R1
            namesf{k} = sprintf('x_{%d}',k);
        end
    end
end


if nargin==3
    sortopts = '';
end
if nargin==2
    sortopts = 'no';
    zoom = 1;
end

if  strcmp(sortopts,'no')
    fprintf('No sort...\n');
    idxc = 1:R2;
    idxf = 1:R1;    
elseif strcmp(sortopts,'c')
    fprintf('Sorting C using 1D-SOM: cols by cols...\n');
    som = inisom(C,{10*size(C,2)});
    som = bsom(C,som,10,2);
    pr  = fproj(C,som); 
    [aux,idx]=sort(pr);
    idxc = idx;
    idxf = 1:R1;
elseif strcmp(sortopts,'f')
    fprintf('Sorting C using 1D-SOM: rows by rows...\n');
    som = inisom(C',{10*size(C,1)});
    som = bsom(C',som,10,2);
    pr  = fproj(C',som); 
    [aux,idx]=sort(pr);
    idxc = 1:R2;
    idxf = idx;
elseif strcmp(sortopts,'cf')
    fprintf('Sorting C using 1D-SOM: cols by cols and rows by rows...\n');
    som = inisom(C,{10*size(C,2)});
    som = bsom(C,som,10,2);
    pr  = fproj(C,som); 
    [aux,idx]=sort(pr);
    idxc = idx;        
    som = inisom(C',{10*size(C,1)});
    som = bsom(C',som,10,2);
    pr  = fproj(C',som); 
    [aux,idx]=sort(pr);
    idxf = idx;        
else 
    fprintf('Sorting C using 1D-SOM: rows&cols by cols...\n');
    som = inisom(C,{10*length(C)});
    som = bsom(C,som,10,2);
    pr  = fproj(C,som); 
    [aux,idx]=sort(pr);
    idxc = idx;
    idxf = idx;        
end    
C = C(idxf,idxc);
namesc = namesc(idxc);
namesf = namesf(idxf);       
fprintf('...done.\n');    



ang = -90;

if R2<20
    fontsizex = 10;
elseif R1<40
    fontsizex = 8;
else
    fontsizex = 6;
end
if R1<20
    fontsizey = 10;
elseif R1<40
    fontsizey = 8;
else
    fontsizey = 6;
end


% if R1==R2               % We assume that a square matrix is a correlation matrix
%     CLims = [-1 1];
% else                   
%     CLims = [min(C(:)) max(C(:))]
% end

CLims = [-1 1];
imagesc(C,CLims);
colorbar;
set(gca,'xtick',[],'ytick',[]);


fontsizex = 8;
fontsizey = 8;
ang = -90;



for k = 1:R2
    hx = text(k,R1+0.5,namesc(k));
    set(hx,'rotation',ang,'horizontalalignment','left','fontsize',fontsizex);
end

for k = 1:R1
    hy = text(0.5,k,namesf(k));
    set(hy,'horizontalalignment','right','fontsize',fontsizey);
end



if zoom
    while 1,                
        % Get two points for zoom
        [x,y,b]=ginput(2);
        if (length(b)==0)
            return
        end        
        
        nx = round(sort(x));
        ny = round(sort(y));
        
        % Zoom Out
        if b(1)~=1
            nx = [1 size(C,2)];
            ny = [1 size(C,1)];
        end
        
        % Indices of horizontal and vertical axes
        ix = nx(1):nx(2);
        iy = ny(1):ny(2);
        Cplot = C(iy,ix);
        [R1,R2]=size(Cplot);
        
        if R2<20
            fontsizex = 10;
        elseif R1<40
            fontsizex = 8;
        else
            fontsizex = 6;
        end
        
        if R1<20
            fontsizey = 10;
        elseif R1<40
            fontsizey = 8;
        else
            fontsizey = 6;
        end
        
        
        imagesc(Cplot,CLims);
        colorbar;
        set(gca,'xtick',[],'ytick',[]);
        for k = 1:R2
            hx = text(k,R1+0.5,namesc(ix(k)));
            set(hx,'rot',ang,'horizontalalignment','left','fontsize',fontsizex);
        end
        
        for k = 1:R1
            hy = text(0.5,k,namesf(iy(k)));
            set(hy,'horizontalalignment','right','fontsize',fontsizey);
        end
    end
end
