function [smap,MSE] = ekisom(p,smap,epochs,Nc,weights,Bidx)

% EKISOM Variant of the SOM algorithm improving density estimation
%
% CALLS
%       [som,mse] = ekisom(p)
%       [som,mse] = ekisom(p,smap)
%       [som,mse] = ekisom(p,smap,epochs,Nc)
%       [som,mse] = ekisom(p,smap,epochs,Nc,weights)
%       [som,mse] = ekisom(p,smap,epochs,Nc,weights,Bidx)
%
% INPUTS
%       p: Input data matrix (R x Q) where each column correspond to each sample vector
%          in the input space.
%    smap: SOM structure (see INISOM)
%  epochs: Number of times (iterations) which the whole input data set is used for 
%          training
%      Nc: Neighbourhood. It can be either one of these:
%            - a single scalar specifying the last value of a monotonically decreasing
%              neighbourhood.
%            - a (1 x 2) vector with the starting and finishing value of the neighbourhood
%            - a (1 x epochs) vector with values of the neighbourhood at each epoch
% weights: Weight values given to all the samples (by default all samples have the same 
%          weight)
% Bidx:    Index vector containing variables to be taken into account in the 
%          self-organization process
% 
% REFERENCES
% Eider Arsuaga Uriarte, "Ekisom: Un modelo neuronal autoorganizativo para
% minera de datos", PhD Thesis, University of Deusto, Nov. 2007
%
%
%

%     EKISOM Variant of the SOM algorithm improving density estimation
%     Copyright (C) 2009  Ignacio Daz Blanco
%     Area de Ingeniera de Sistemas y Automtica
%     Universidad de Oviedo
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
if nargin == 1
    smap = inisom(p);
end

if nargin <= 2
    epochs = 50;
    Nc = 1;
end


Q   = size(p,2);

% Neighbourhood selection
lnc = length(Nc);
if lnc==1
    neigh = logspace(log10(max([cat(1,smap.dims{:})]/3)),log10(Nc),epochs);
elseif lnc==2
    neigh = logspace(log10(Nc(1)),log10(Nc(2)),epochs);
elseif lnc>2
    if lnc~=epochs
        error('length of neighborhood vector does not match number of epochs');
    end;
    neigh = Nc;
end;



STD2 = max(eig(cov(p'))),       % Compute the maximum variability
mu  = 50;                       % Weight for the input distance in the SOM cooperation

% Batch SOM algorithm
if nargin<6,
    Bidx = 1:smap.R;
end

fprintf('Ejecutando algoritmo EKISOM...\n');
for ke = 1:epochs
    [m,bmu] = min(matdist(smap.w(:,Bidx)',p(Bidx,:)),[],1); % Find the bmu for all input data
    
    dis_inp = double(smap.dis(:,bmu)).^2;                   % Distance in the input space
    dis_out = matdist(smap.w',smap.w(bmu,:)')/STD2;         % Distance in the output space (normalized by STD2)
  
    hci = exp(-(dis_inp + mu*dis_out) / (2*neigh(ke).^2));  % EKISOM neighborhood,. Cfr (Arsuaga, 2007)
        
    if nargin==5
        hci   = hci.*(ones(smap.S,1)*weights);
    end
    
    smap.w = (hci*p')./(sum(hci,2)*ones(1,smap.R));         % Main equation of SOM batch training
    
    MSE(ke) = mean(m);
    if ~rem(ke,epochs/10)
        fprintf('Epoca: %d,\t Vecindad: %f,\t mse: %f\t mse (p.u.): %f \n',ke,neigh(ke),MSE(ke), MSE(ke)/MSE(1));
    end
    
    
end