function Phi = exprbf(Xi,Xj,Cj,order)
% EXPRBF Gaussian radial basis function
%
%        Phi_ij = exprbf(Xi,Xj,Cj)
%
% INPUTS
%   Xi: Points in the joint space by columns (RxQ)
%   Xj: centers of the gaussians (RxNc)
%   Cj: N-Array of covariance matrix (RxRxNc)
%order: Putting something here adds a column of 1's in Phi to 
%       allow for a bias term in RBF Neural Networks
%
% OUTPUTS
%   Phi_ij: Matrix containing the computatins of the 
%           exp() function (QxNc)
%
% Obtains in a matrix the values of a multivariate exp() function. The
% difference with MVG is that this one is not normalized to unit volume.
%
% See also MVG

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


D  = size(Xi,1);
Ni = size(Xi,2);
Nj = size(Xj,2);


if length(Cj)==1
   dis = matdist(Xi,Xj);
   Phi = exp(-dis/Cj);   
elseif size(Cj,3)==1
   dis = matdist(Xi,Xj);
   Phi = exp(-dis./(ones(Ni,1)*Cj(:)'));   
else
	[j,i]=meshgrid(1:Nj,1:Ni); 
	j = j(:)';
	i = i(:)'; 

	Xij   = Xi(:,i)-Xj(:,j);
	CjXij = zeros(D,Ni*Nj);
   
   for j = 1:Nj
   k = j*Ni-Ni+1:j*Ni;
   CjXij(:,k) = inv(Cj(:,:,j))*Xij(:,k);
	end

Phi=exp(-reshape(sum(Xij.*CjXij,1),Ni,Nj));
end



if nargin==4
    % Sugeno model type 0 without bias term
    if length(order)==0     
        return;
    % Sugeno model type 0 with bias term    
    elseif order==0
        Phi = [Phi ones(Ni,1)];
    % Sugeno model type 1 with bias term  
    elseif order==1
        aux = [];
        for k = 1:Ni
            aux(k,:) = kron([Phi(k,:) 1],[Xi(:,k)' 1]);
        end
        Phi = aux;
    elseif order>1
        error('Local model orders higher than 1 are not allowed');
    end
    
end
