function [c,mf] = fcms(x,Nc,epochs,expo,pdfx)
% FUZZY C-MEANS: Fuzzy c-means algorithm for unsupervised classification
%
%         c = fcms(x,Nc,epochs,pdfx)
%
% Inputs
%      x: Input data matrix with points by columns (RxQ)
%     Nc: Number of clusters (scalar)
% epochs: (Optional) Number of iterations (Default = 100)
%   expo: (Optional) Exponent in the original Bezdek algorithm (Default = 2)
%   pdfx: (Optional) Probability density function values associated to each
%          input data point (1xQ) (Default = uniform)
%
% Outputs
%      c: Matrix with the centers by columns (RxNc)
%
% Calls
%        c = fcms(x,Nc)
%        c = fcms(x,Nc,epochs)
%        c = fcms(x,Nc,epochs,expo)
%        c = fcms(x,Nc,epochs,expo,pdfx)
%
% This function implements the Bezdek's fuzzy c-means algorithm. Additionally, a 
% probability value can be associated to each data sample to generate a distribution 
% of points with a given probability density function.
%
% NOTE: The algorithm performs well in all dimensions if data has a cluster
% structure (multimodality). However, when dimension is high, and the number of
% clusters is lower than the number of prototypes the algorithm places all the
% prototypes in the two clusters. The algorithm does not achieve a magnification
% factor proportional to the density of the data. This nice property only occurs
% for low dimensions (below 5). By selecting an exponent closer to 1, this property
% is more closely achieves for high dimensions, but still fails for data dimensions
% above 20D.

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.





[R,N]=size(x);

% Random initialization of centers
% c = 2*rands(R,Nc);

% PCA initialization
mx   = mean(x,2);
xdet = x-mx*ones(1,N);
Cxx  = xdet*xdet'/(N-1);
c    = 1.1*sqrtm(Cxx)*randn(R,Nc);





% Default number of epochs
if nargin == 2
    epochs = 100;
    expo = 2;
elseif nargin == 3
    expo = 2;
end

% Algorithm
for k = 1:epochs
    dst     = matdist(x,c);  
    phi     = dst.^(-1/(expo-1))';    
    phinorm = phi./(ones(Nc,1)*sum(phi,1));
    mf      = phinorm.^(expo);    
    % Assignment of probability weights
    if nargin == 5
        mf = mf.*(ones(Nc,1)*pdfx(:)');
    end
    c  = (mf*x')'./(ones(R,1)*sum(mf,2)');
    if ~rem(k,ceil(epochs/10))
        [qerr,bmu]=min(dst');
        fprintf(['Epoch: ' num2str(k) '/' num2str(epochs) '\t qerr (MSE) = ' num2str(mean(qerr)) '\n']);
    end
end

