function [pr,res,bmu] = fproj(p,smap,projtype)

% FPROJ Forward Projection using the SOM
%
% [pr,res,bmu] = fproj(p,smap,projtype)
%
%        p: (R x Q) vector with data points by columns
%     smap: SOM struct (see INISOM, BSOM)
% projtype: String with the projection type:
%           'discrete': Original discrete SOM projection (default)
%           'backproj': SOM resampling followed by discrete SOM projection.
%           'grnn':     Generalized Regression Neural Network
%           'ls':       Uses seast squares (Ridge Regression) to compute 
%                       the weight matrix
%
% CALLS
%           [pr,res,bmu] = fproj(p,smap)
%           [pr,res,bmu] = fproj(p,smap,projtype)
%
% DESCRIPTION
%    Projects input space samples given as columns of data matrix p) onto
%    the visualization space (grid space) using the projection defined by
%    the som 'smap'. Projection can be either discrete or continuous.
%
% See also BPROJ

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



if nargin==3    
    % Continuous SOM projection
    if strncmp(projtype,'discrete',1)
        % Discrete SOM projection
        [qerr,bmu] = min(matdist(smap.w',p));
        pest = smap.w(bmu,:)';
        pr = smap.pos(:,bmu);
    
    elseif strncmp(projtype,'backproj',1)
        for k = 1:length(smap.dims)
            dimi{k} = smap.dims{k}*3;
            if dimi{k} > 50
                dimi{k} = 50;
            end
        end
        u = rectopi(smap.dims{:},dimi{:});
        grnnbwd = grnntra(smap.w',smap.pos,2,0);
        wi = grnnsim(u,grnnbwd);
        [qerr,bmu] = min(matdist(wi,p));
        pest = wi(:,bmu);
        pr = u(:,bmu);
        
    elseif strncmp(projtype,'grnn',1)
        % Continuous Projection using GRNN
        grnnfwd = grnntra(smap.pos,smap.w');                % Forward projection GRNN
        grnnslf = grnntra(smap.w',smap.w');                 % Self projection GRNN
        pr   = grnnsim(p,grnnfwd);
        pest = grnnsim(p,grnnslf);
    elseif strncmp(projtype,'ls',1)
        % Continuous Projection using LS weight matrix computation
        grnnfwd = grnntra(smap.pos,smap.w',0.0001,'e');     % Forward projection GRNN
        grnnslf = grnntra(smap.w',smap.w',0.0001,'e');      % Self projection GRNN
        pr   = grnnsim(p,grnnfwd);
        pest = grnnsim(p,grnnslf);
    end
    
    
else
    % Discrete SOM projection
    [qerr,bmu] = min(matdist(smap.w',p));
    pest = smap.w(bmu,:)';
    pr = smap.pos(:,bmu);
end

% Computation of Residuals
res = p - pest;