function [yi,act] = grnnsim(xi,grnn,idx)
% GRNNSIM Simulate a GRNN with autoassociative completion
%   [yi,act] = grnnsim(xi,grnn,idx)
%
% INPUTS
%   xi: (Rin x Q) data in the input space. This vector can contain NaN values 
%       for which the GRNN performs autoassociative completion
% grnn: (struct) a GRNN object generated by grnntra()
%  idx: Index vector with the variables of xi to be taken into account.
%
% OUTPUTS
%   yi: (Rout x Q) data in the output space
%  act: (1xQ) is the activation level of the bmu for each sample.
%
% DESCRIPTION
%   Simulates a previously trained GRNN. NaN values can be inserted in
%   data matrix xi to account for unavailable or not valid data. The GRNN
%   will perform autoassociative completion for these values.
%
% See also GRNNTRA

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


% Retrieve net parameters
if nargin == 2
    m = grnn.m;
elseif nargin == 3          % If some inputs are not to be considered ...
    xi = xi(idx,:);         % ... take only those which are concerned
    m  = grnn.m(idx,:);     %
end
C = grnn.C;             % widths of the receptive fields
W = grnn.W;             % weights from hidden to output layer



Q    = size(xi,2);
dimy = size(W,2);       

% Compute the activation of the receptive fields
psi  = psirbf(xi,m,C,grnn.order); 

% Compute outputs
yi   = psi*W;
yi   = yi';

% Compute the activation level of the best matching unit
act = max(exprbf(xi,m,C),[],2)';
