function grnn = grnntra(y,x,C,L,centers,order)
% GRNNTRA Generalized Regression Neural Network Train
%
%   grnn = grnntra(y,x,C,L,centers,order)
%
% INPUTS
%       y: output training points (Ry,N)
%       x: input  training points (Rx,N)
%       C: kernel parameters (kernel widths)
%       L: Weight decay regularization parameter
% centers: Model centers
%   order: Sugeno model order
%
%
% Trains the GRNN and produces the following structure
%
%     grnn.m: RBF centers
%     grnn.C: RBF kernel parameters
%     grnn.W: RBF weights
% grnn.order: Sugeno order of the approximation
%
% Other call conventions:
%
%   grnn = grnntra(y,x)             Automatically computes kernel parameters and weights
%   grnn = grnntra(y,x,C)           Original Spetch's GRNN algorithm (no least squares weights) 
%   grnn = grnntra(y,x,C,L)         Uses least squares interpolation with regularization (weight decay factor L). 
%   grnn = grnntra(y,x,L,'e')       Same as above but computes kernel parameters automatically
%   grnn = grnntra(y,x,C,L,centres) Uses 'centres' instead of 'x'
%
% See also GRNNSIM

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


K = 10;                 % Number of nearest neighbours
dimx = size(x,1);
dimy = size(y,1);
S    = size(x,2); 
threshold = 0.000;      % Threshold to avoid almost-zero kernel sizes

if size(y,2)~=size(x,2)
    error('Number of samples of input and output data are not equal')
end


if nargin<=5
    order = [];
end

if nargin==2
    % disp(['Heuristic computation of kernel widths (RMS distance of ' num2str(K) 'KNN)...']);
    D = sort(matdist(x,x));
    % C = mean(mean(sqrt(D(2:6,:))))*dimx;
    C = sqrt(mean(D(2:K,:))) + threshold;
    % disp('Computation of GRNN weights...');
    W = y';
    m = x;
elseif nargin == 3
    % disp('Computation of GRNN weights...');
    W = y';
    m = x;
elseif nargin == 4
    if isstr(L)
        % the third parameter is now lambda
        L = C; 
        % disp(['Heuristic computation of kernel widths (RMS distance of ' num2str(K) 'KNN)...']);
        D = sort(matdist(x,x));    
        % C = mean(mean(sqrt(D(2:6,:))))*dimx;
        C = sqrt(mean(D(2:K,:))) + threshold;        
    end
    % disp('Computation of exact GRNN weights...');
    % Least Squares computation of weights
    phig = psirbf(x,x,C,order);
    W = inv(phig'*phig + L*eye(size(phig,2)))*phig'*y';
    m = x;
    
elseif nargin>=5
    % fprintf('Computation of exact GRNN weights...\n');
    % fprintf('Sugeno Order: %f',order);
    phig = psirbf(x,centers,C,order);
    W = inv(phig'*phig + L*eye(size(phig,2)))*phig'*y';
    m = centers;    
end

grnn.m = m;
grnn.C = C;
grnn.W = W;
grnn.order = order;

