function smap = inisom(p,dims,featnames,limits);

% INISOM2 Initialize Self Organizing map
%
%            smap = inisom(p,dims,featnames)  
%
% INPUTS
%         p: Data vector (R x Q)
%      dims: Dimensions of the SOM {1 x NumDims}
% featnames: Names of the R features considered {1 x R}
%
% OUTPUTS
%      smap: Structure with SOM parameters (Dimensions, 
%            grid positions, codebook vectors,...)
%
% DESCRIPTION
%            Initializes a SOM structure for further training and 
%            execution. Initial codebook vectors are chosen using
%            PCA initialization to cover the data span.
%
%
% EXAMPLES
%            % 2D-SOM example (most typical case)
%            p     = [randn(3,100)-3 randn(3,100)+3];
%            som   = inisom(p,{20,20},{'x','y','z'});
%            som   = bsom(p,som,10,1);
%            plotsom(p,som); planes(som);
%
%            % 1D SOM example
%            p     = [randn(3,100)-3 randn(3,100)+3];
%            som1D = inisom(p,{50},{'x','y'});
%            som   = bsom(p,som,10,1);
%            plotsom(p,som);
%
%            % 3D SOM example
%            p     = [randn(3,100)-3 randn(3,100)+3];
%            som3D = inisom(p,{7,7,7},{'x','y'});
%            som   = bsom(p,som,10,1);
%            plotsom(p,som); planes3d(som);
%
% See also BSOM, FPROJ, BPROJ

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



R = size(p,1);
Q = size(p,2);



% Set default SOM dimensions if not specified
if nargin == 1
    if Q > 1000
        Ni = round(20*sqrt(1000/Q));
        Nj = round(20*sqrt(1000/Q));
    else
        Ni = 20;
        Nj = 20;
    end
    dims = {Ni,Nj};
end

% if length(dims)==1
%     dims{2} = 1;
% end

pos = rectop(dims{:});
dis = distenlace(pos);
S   = length(pos);


dimvec = cat(2,dims{:});


% Do not use PCA initialization if SOM size is too large
if S>3600
    PCAinit = 0;
else
    PCAinit = 1;
end



if PCAinit == 1
    if R > 1
        disp('PCA Initialization...');
        posm = detrend(pos',0)';
        pm   = detrend(p',0)';
        meanpoint = mean(p')';
        [u,s,v] = svd(pm*pm'/Q);
        uortho = (u*sqrt(s));
        
        % CORREGIDO BUG: 2009-07-22
        % La inicializacin basada en PCA no recubria los datos en algunos
        % casos debido a que no se asignaba el vector principal a la
        % dimension mas grande de la reticula... cuando una de las
        % dimensiones era 1 y el vector principal coincidia con dicha
        % dimension, sencillamente la reticula no se expandia en la
        % direccion principal sino en la secundaria... con esta correccion
        % deberian asignarse las dimensiones mas grandes a los vectores
        % principales...
        [aux,idx]=sort(-cell2mat(dims));
        w = ((uortho(:,idx)*diag(1./dimvec))*posm*2*2)' + (meanpoint*ones(1,S))';        
        %w = ((uortho(:,1:length(dims))*diag(1./dimvec))*posm*2*2)' + (meanpoint*ones(1,S))';
        % F_CORREGIDO
        
    else
        aux = linspace(min(p),max(p),dims{2});
        w = rectop(aux,zeros(1,dims{1}),'idx')';        
        w = w(:,1);
    end
else
    disp('Random Initialization...');
    [aux,npar] = normalize_data('mnmx',p);
    %w = normalize_data('reverse',rands(R,S),npar);
    w = normalize_data('reverse',2*rand(R,S)-1,npar);
end


% Set default labels for each feature
if ~exist('featnames') || length(featnames)==0;
    for k = 1:R
        featnames{k} = ['Feature ' num2str(k)];
    end
end

% Set default limit values
if ~exist('limits')
    limits = minimaxi(p);
end


% Build the SOM struct
smap.dims= dims;
smap.R   = R;
smap.S   = S;
smap.pos = pos;
smap.dis = dis;
smap.w   = w;
smap.featnames = featnames;  
smap.limits = limits;


