function [p2,eigenvalues,idxcomp1,dx]=isomap(p,dim,K)
% ISOMAP: Tenenbaum's ISOMAP algoritm
% 
%         [p2,eigenvalues]=isomap(p,dim)
%
% p:   Data points in the input space (RxQ)
% dim: Dimension of the embedding space
%
% This function implemenes [Tenenbaum00] isomap algorithm
% for dimension reduction. The key idea of the algoritm is
% the preservation of the geodesic lines (shortest paths)
% in the input space
%
% [Tenenbaum00] "A Global Geometric Framework for Nonlinear 
%               Dimensionality Reduction" Science, Vol. 290 
%               22 December 2000

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


N = size(p,2);
R = size(p,1);


% Step 1: Construct neighborhood graph
if N==R
    dx  = p;                            % If p is squared, we suppose it is already a distance matrix
else
    dx  = sqrt(matdist(p,p));           % Else, get the matrix of distances
end




% KNN-Isomap
[dxsort,idxsort] = sort(dx);
for j = 1:N
    % fprintf('.');
    dx(idxsort(K+1:N,j),j) = inf*ones(N-K,1);
end

dx = min(dx,dx');                         % Make sure distance matrix is symmetric (cfr. Tenenbaum)


% Epsilon-Isomap
% epsilon = max(min(dx+inf*eye(N)));      % Get a suitable value of epsilon
% idx     = find(dx(:) > epsilon);        % Find the links between points
% dx(idx) = inf*ones(length(idx),1);      % Set infinite distances for not linked nodes



% Step 2: Compute shortest paths
dg = floyd(dx);                         % Compute the geodesics (minimum paths) in the input space




if sum(isinf(dg(:)))>0
    disp('Non-conex graph. Splitting into conex components...');
    [foo,idx] = min(dg==inf);
    [comps,foo,numpts] = unique(idx);
    NumComps = size(comps);
    disp(['Number of conex regions (components): ' num2str(NumComps)]);
    disp('We take just the elements of component 1');
    idxcomp1  = find(numpts==1);             % we take the elements of the first component
    dg = dg(idxcomp1,idxcomp1);
    dx = dx(idxcomp1,idxcomp1);
    N  = length(idxcomp1);
else
    idxcomp1 = 1:N;
end


% Step 3: Construct d-dimensional embedding
H = eye(N)-1/N;                         % Compute centering matrix
tauD = -H*(dg.^2)*H/2;                  % Apply operator tau(Dg)
[v,d]=eig(tauD);                        % Reduce dimensionality using typical PCA 
p2 = sqrt(d(1:dim,1:dim))*v(:,1:dim)';  % 

eigenvalues = diag(d);


