function mloc = mloc_loc(y,ny,u,nu,p,mloc,usar_bias,sigma)
% MLOC_LOC Entrena los modelos locales de un SOM 
%  
%   CALLS
%         [mloc] = mloc_loc(y,ny,u,nu,p,mloc)
%         [mloc] = mloc_loc(y,ny,u,nu,p,mloc,usar_bias,sigma)
%  
%   INPUTS
%         y: Matriz de datos de salida (Ry x Q)
%        ny: Vector de retardos en la parte AR del modelo
%         u: Matriz de datos de entrada (Ru x Q)
%        nu: Vector de retardos en la parte MA del modelo
%         p: variables exgenas del vector de ndices (Rp x Q)
%      mloc: Estructura SOM modelando el espacio engendrado por las variables selectoras
% usar_bias: Emplear trmino de bas en la estimacin (defecto, 1)
%     sigma: Amplitud (radio) del entorno local sobre el que se construyen los modelos
%
%
%   OUTPUTS
%      mloc: Estructura de SOM con modelos locales

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


% Se recuperan algunos parmetros obtenidos en la etapa anterior
nq  = mloc.nq;
np  = mloc.np;
A   = mloc.A;
b   = mloc.b;
som = mloc.som;
Ry  = mloc.Ry;
lny = length(ny);
lnu = length(nu);




if nargin==6
    usar_bias = 1;
    sigma = 8;
elseif nargin == 7
    sigma = 8;
end
    

% Se crean vectores "augmented"
idx = max([ny nu np nq])+1:length(y);

% Variable "augmented" de salida del proceso
for i = 1:Ry
    phiy{i}=[];
    for j = 1:lny
        phiy{i} = [phiy{i};y(i,idx-ny(j))];
    end
end

% Variable "augmented" de entrada al proceso
phiu = [];
for j = 1:lnu
    phiu = [phiu;u(:,idx-nu(j))];
end

% VARIABLES SELECTORAS DE DINMICA
% Al considerar retardos incluimos la propia dinamica de la variable como
% elemento de seleccion en la dinamica del proceso. Como caso particular,
% haciendo p = y, tenemos el caso de procesos no lineales descrito por [Cho
% y Principe]


% Inicializacin
phip = [];

% Parte endgena
for j = 1:length(nq)
    phip = [phip;y(:,idx-nq(j))];
end

% Parte exgena
for j = 1:length(np)
    phip = [phip;p(:,idx-np(j))];
end












pp = norpca(phip,A,b);



% 2007-12-07: para el caso multivariable (cuando los vectores de y
% tienen dimensin mayor que 1, la variable ytot debe contener vectores de
% tamao Ry x Q

% ytot = y(idx);        % Codigo anterior
ytot = y(:,idx);






% ESTIMACIN DEL MODELO LINEAL GLOBAL 

% ANTIGUO: ESTE VALOR DEL PARMETRO DE REGULARIZACIN DEGRADABA DEMASIADO LA SOLUCIN
% lambda = 1e-5;

% 2010-07-06: 
% El parmetro de regularizacin en ocasiones degrada demasiado
% la solucin. Se comprob en algunos casos que bajarlo a 1e-9 o incluso
% dejarlo a cero produca los resultados esperables. Por el contrario, no
% se han constatado apenas casos en los que sea necesario regularizar la 
% solucin mediante un valor distinto de cero. Por tanto, se deja este valor a cero
% por defecto.
lambda = 0;





% Trmino de bias. Permite computar un modelo afn,
% es decir, con un parmetro independiente que
% permite considerar medias no nulas
if usar_bias
    bias = ones(1,length(idx));
else
    bias = [];
end


% OBSOLETO
% disp('Determinando modelo lineal global a efectos de comparacin...');
% mloc.Tlineal = mloc_glo(y,u,ny,nu,usar_bias);
% F_OBSOLETO

% 2008-09-02
% Se calcula una lnea de T para cada elemento de y (i.e. para cada salida) para
% evitar incluir trminos cruzados (dependencias respecto a muestras anteriores de las restantes
% salidas)

% El vector de ndices "idx" contiene todos los ejemplos
if length(phiu)
    % Se obtienen los coeficientes del modelo para todos los ejemplos
    for i = 1:Ry,
        [Taux,rmse_aux(i)] = linreg(ytot(i,:),[phiy{i};phiu;bias],lambda);
        Tlineal(i,:) = [kron(Taux(1,1:lny),[zeros(1,i-1) 1 zeros(1,Ry-i)]), Taux(1,lny+1:end)];
    end
else
    % caso particular cuando phiu
    % est vaco (el modelo es AR puro y no hay entradas)
    for i = 1:Ry
        [Taux,rmse_aux(i)] = linreg(ytot(i,:),[phiy{i};bias],lambda);
        Tlineal(i,:) = [kron(Taux(1,1:lny),[zeros(1,i-1) 1 zeros(1,Ry-i)]), Taux(1,lny+1:end)];
    end
end

mloc.Tlineal = Tlineal;




% ESTIMACIN DE LOS MODELOS LOCALES
% Para cada neurona (modelo local) el vector de ndices "idx" contiene 
% ejemplos de la vecindad de dicha neurona
pr = fproj(pp,som);
for k = 1:som.S, 
    if ~rem(k,50)
        fprintf('.');
    end

    distancias = sqrt(matdist(pr,som.pos(:,k)));    % obtenemos las distancias de la neurona actual (cuyo modelo local se va a calcular) 
    % a cada bmu del vector de ndices de modelo phip
    idxloc = find(distancias < sigma);                 % nos quedamos slo con las neuronas cuya distancia es inferior a una dada (ej. 10)

    % Trmino de bias. Permite computar un modelo afn,
    % es decir, con un parmetro independiente que
    % permite considerar medias no nulas
    if usar_bias
        bias = ones(1,length(idxloc));
    else
        bias = [];
    end

    % 2006-09-22
    % Nota: en la versin mltra_v01 se consideraban pesos para cada dato de acuerdo con la proximdad 
    % de la bmu a la neurona actual. El mtodo era mucho ms lento y parece que apenas se mejora el 
    % resultado. El procedimiento seguido aqu es una especie de "proximidad burbuja"


    % 2008-09-02
    % Se calcula una lnea de T para cada elemento de y (i.e. para cada salida) para
    % evitar incluir trminos cruzados (dependencias respecto a muestras anteriores de las restantes
    % salidas)
                                                
    if length(phiu)
        % Se obtienen los coeficientes del modelo
        % de los datos cuyas
        % bmu caen en un
        % entorno prximo
        % para la neurona actual a partir
        for i = 1:Ry,
            [Taux,rmse_aux(i)] = linreg(ytot(i,idxloc),[phiy{i}(:,idxloc);phiu(:,idxloc);bias],lambda);
            T(i,:) = [kron(Taux(1,1:lny),[zeros(1,i-1) 1 zeros(1,Ry-i)]), Taux(1,lny+1:end)];
        end
    else
        % caso particular cuando phiu
        % est vaco (el modelo es AR puro y no hay entradas)
        for i = 1:Ry
            [Taux,rmse_aux(i)] = linreg(ytot(i,idxloc),[phiy{i}(:,idxloc);bias],lambda);
            T(i,:) = [kron(Taux(1,1:lny),[zeros(1,i-1) 1 zeros(1,Ry-i)]), Taux(1,lny+1:end)];
        end
    end

    Mcell{k} = T;
    % La matriz T contiene los parmetros del modelo
    % local asociado a cada neurona. Dichos
    % parmetros son los coeficientes de la
    % funcin de transferencia entre entrada y
    % salida (ms el parmetro de bias, si lo hay)
    RMSE(k) = sqrt(sum(rmse_aux.^2));
    rango(k) = 0;
end;


fprintf('\n');



% GUARDANDO LOS DATOS EN UN OBJETO TIPO mloc
mloc.nu = nu;                       % Nmero de retardos de los modelos ARX (parte X)
mloc.ny = ny;                       % Nmero de retardos de los modelos ARX (parte AR)
mloc.Mcell = Mcell;                 % Cell array con los modelos ARX asociados a cada prototipo
mloc.usar_bias = usar_bias;         % flag de uso de trmino bias
mloc.rmse = RMSE;                   % plano con los errores de estimacin
mloc.rango = rango;
mloc.sigma = sigma;
mloc.Ru    = size(u,1);             % dimensin de los vectores de entrada.


