function [yest,pr] = mloc_sim(u,mloc,p,y0)
% MLOC_SIM Simula un SOM de modelos locales
%
%   CALLS
%         [yest,pr] = mloc_sim(u,mloc,p,y0)
%
%   INPUTS
%         u: Matriz de datos de entrada (R x Q)
%      mloc: Estructura de SOM con modelos locales
%         p: Matriz de datos con variables ndice exgenas selectoras (Rp x Q)
%        y0: Matriz de datos con los valores iniciales conocidos de y
%
%   OUTPUTS
%      yest: Matriz de datos estimados
%       bmu: Vector de neuronas ganadoras
%
%   Nota:
%           El algoritmo toma como valores iniciales (suponiendo ny>=nu)
%           [y0(k-1) y0(k-2) ... y0(k-ny)]
%           [ u(k-1)  u(k-2) ...  u(k-nu)]
%           
%           y devuelve
%           [yest(1) yest(2) ... yest(k-ny),    yest(k-ny+1) ... yest(Q)]
%           -------------------------------     ------------------------
%              valores no predichos                  predicciones
%
%           Es decir, las verdaderas "predicciones" estn a partir del elemento ny+1 de yest de 
%           manera que:
%
%           yest(ny+1) es el "one step prediction", conocidos los datos y0(1:ny), u(1:ny)
%           yest(ny+2) es el "two-step prediction", conocidos los datos y0(1:ny), u(1:ny)
%
%           etc...

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



nu = mloc.nu;
ny = mloc.ny;
np = mloc.np;
nq = mloc.nq;


Mcell = mloc.Mcell;
A = mloc.A;
b = mloc.b;
som = mloc.som;
usar_bias = mloc.usar_bias;




yest = zeros(size(mloc.Mcell{1},1),size(u,2));
yglo = zeros(size(mloc.Mcell{1},1),size(u,2));

if nargin==4
    yest(:,1:size(y0,2)) = y0;
    yglo(:,1:size(y0,2)) = y0;
end


for k = max([ny nu np nq])+1:length(u)
    phiy = [];
    for j = 1:length(ny)
        phiy = [phiy;yest(:,k-ny(j))];      % Construimos el vector "augmented" de salida con las salidas estimadas anteriores
    end

    phiu = [];
    for j = 1:length(nu)
        phiu = [phiu;u(:,k-nu(j))];         % Construimos el vector "augmented" de entrada con las entradas anteriores
    end

    % Inicializacin
    phip = [];

    % Parte endgena
    for j = 1:length(nq)
        phip = [phip;yest(:,k-nq(j))];     
    end

    % Parte exgena
    for j = 1:length(np)
        phip = [phip;p(:,k-np(j))];     
    end

    pp = norpca(phip,A,b);


    [qerr,bmu]=min(matdist(som.w',pp));     % Buscamos en el mapa SOM el modelo local para la muestra actual
    pr(k) = bmu;
    Tthis = Mcell{bmu};
    if usar_bias
        yest(:,k) = Tthis*[phiy;phiu;1];      % Estimacion mediante el modelo local actual (usando trmino bias)
    else
        yest(:,k) = Tthis*[phiy;phiu];        % Estimacion mediante el modelo local actual (sin usar trmino bias)
    end
end
