function mloc = mloc_som(y,nq,p,np,dims,epochs,Nc)
% MLOC_SOM Entrena un SOM en el espacio "embedded" para segmentar dinmicas
%  
%   CALLS
%       mloc = mloc_som(y,nq,p,np)
%       mloc = mloc_som(y,nq,p,np,dims,epochs,Nc)
%  
%   INPUTS
%         y: Matriz de datos de salida (Ry x Q)
%        nq: Vector de retardos de y en el vector de ndices (parte endgena del vector de ndices)
%         p: variables exgenas del vector de ndices (Rp x Q)
%        np: Vector de retardos de p en el vector de ndices (parte exgena  del vector de ndices)
%      dims: Tamao del som (valor por defecto, {30,30})
%    epochs: Nmero de pocas (valor por defecto, 10)
%        Nc: Vecindad final (valor por defecto, 0.4)
%
%      Nota: Un vector de variables ndice tpico tiene la siguiente estructura
%               phip = [y(k-n1) y(k-n2)... y(k-nq), p(k-n1)... p(k-np)]
%                       ------------------------    ------------------
%                            parte endgena           parte exogena
%
%
%   OUTPUTS
%      mloc: Estructura de SOM modelando el espacio engendrado por phip

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



if nargin==4
    dims = {30,30};
    epochs = 10;
    Nc = 0.4;
end



% Se crean vectores "augmented"
idx = max([np nq])+1:length(y);


% VARIABLES SELECTORAS DE DINMICA
% Al considerar retardos incluimos la propia dinamica de la variable como
% elemento de seleccion en la dinamica del proceso. Como caso particular,
% haciendo p = y, tenemos el caso de procesos no lineales descrito por [Cho
% y Principe]


% Inicializacin
phip = [];

% Parte endgena
for j = 1:length(nq)
    phip = [phip;y(:,idx-nq(j))];
end

% Parte exgena
for j = 1:length(np)
    phip = [phip;p(:,idx-np(j))];
end


% Division del problema en modelos locales mediante SOM
[pp,A,b] = norpca(phip);


som = inisom(pp,dims);
som = bsom(pp,som,epochs,Nc);                       


% GUARDANDO LOS DATOS EN UN OBJETO TIPO mloc
mloc.np = np;                       % Nmero de retardos de las variables selectoras endgenas
mloc.nq = nq;                       % Nmero de retardos de las variables selectoras exgenas
mloc.som = som;                     % SOM entrenado con las variables selectoras de dinmica
mloc.A = A;                         % Matriz de normalizacin PCA
mloc.b = b;                         % Vector de normalizacin PCA
mloc.Ry= size(y,1);                 % Dimensin de los vectores de salida
mloc.Rp= size(p,1);                 % Dimensin de los selectores de dinmica
