function Phi = mvg(Xi,Xj,Cj)

% MVG: Multivariate Gaussian.
%
%       Phi = mvg(Xi,Xj,Cj)
%
% Inputs
%   Xi: Points in the joint space by columns (RxQ)
%   Xj: centers of the gaussians (RxNc)
%   Cj: N-Array of covariance matrix (RxRxNc)
%
% Outputs
%   Phi: Matrix of probability densities (QxNc)
%
% Obtains the densities of a multivariate gaussian density function. The
% difference with EXPRBF is that this one is normalized to unit volume.
%
% See also EXPRBF


%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


R = size(Xi,1);
Q = size(Xi,2);
Nc= size(Xj,2);

if size(Cj(:),1)==1
    dis = matdist(Xi,Xj);
    Phi = (1/(2*pi*Cj)^(R/2))*exp(-0.5*dis/Cj);
elseif size(Cj,1)==1
    dis = matdist(Xi,Xj);
    Phi = (ones(Q,1)*(1./(2*pi*Cj).^(R/2))).*(exp(-0.5*dis./(ones(Q,1)*Cj(:)'))); 
else
    Xij = matdiff(Xi,Xj);
    CjXij = zeros(R,Q*Nc);
    for j = 1:Nc
        k = j*Q-Q+1:j*Q;
        CjXij(:,k) = inv(Cj(:,:,j))*Xij(:,k);
        Knorm(j) = inv((2*pi)^(R/2)*sqrt(det(Cj(:,:,j))));
    end
    Phi=(ones(Q,1)*Knorm).*exp(-0.5*reshape(sum(Xij.*CjXij,1),Q,Nc));
end

   
