function pdf = parzen(m,p,C,normalize)
% PARZEN Estimates joint pdf using parzen kernels
%
%        pdf = parzen(m,p,C,normalize)
%
%         m: matrix (RxQ1) with points where joint pdf has to be evaluated
%         p: data matrix (RxQ2) with points of distribution
%         C: kernel bandwidth (covariance matrices) can be scalar, vector (1xQ2) 
%            of width factors for each point of m or an n-array (RxRxQ2) with a whole
%            covariance matrix for each point of m
% normalize: 1 (default) for minmax normalization, 0 in other case
%
% DESCRIPTION
%   Estimates the joint pdf of data points stored in m, and
%   evaluates it for points of p using a Parzen kernel estimation with
%   gaussian RBF's of widths given by C
%
% EXAMPLES
%   % PDF scatter plot
%   figure;
%   p = getpoints;
%   u = rectop(linspace(-10,10,50),linspace(-10,10,50),'idx');
%   pdf = parzen(u,p,0.1);
%   planes(u,pdf);
%   hold on;
%   plotp(p);
%   hold off;
%
%
%   % Visualization of PDF map
%   figure;
%   p = [randn(3,100)-3 randn(3,100)+3];
%   som = inisom(p,{20,20},{'x','y','z'});
%   som = bsom(p,som,10,3);
%   figure(1);
%   pdf = parzen(som.w',p,0.1);
%   figure(1); 
%   planes(som.pos,pdf,{'low dim manifold pdf map'})

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


if nargin==2
    C = 0.2;
    normalize = 1;
elseif nargin==3
    normalize =1;
end

if normalize
    [pn,pn_npar] = normalize_data('mnmx',p);
    mn = normalize_data('apply',m,pn_npar);
else
    pn = p;
    mn = m;
end

pdf = mean(mvg(mn,pn,C),2)';

