function [hfig,hbar]=planes(arg1,arg2,arg3,arg4,arg5);
% PLANES Draws 2D component planes
%
% CALLS
% [hfig,hbar] = planes(smap)
% [hfig,hbar] = planes(smap,plottype)
% [hfig,hbar] = planes(smap,featset)
% [hfig,hbar] = planes(smap,featset,plottype)
% [hfig,hbar] = planes(u,feat)
% [hfig,hbar] = planes(u,feat,featnames)
% [hfig,hbar] = planes(u,feat,plottype)
% [hfig,hbar] = planes(u,feat,featnames,plottype)
% [hfig,hbar] = planes(u,feat,featnames,plottype,limits)
%
% Notation:
%         P = Number of features
%         S = Number of lattice points
%
% smap:      Som structure
% plottype:  String specifying the type of plot. Can be either:
%               'image':   Colormap image plot type (default)
%               'contour': Contour line plot type
%               'surface': Surface plot type
% featset:   Set of features to be plotted. Can be specified either as an
%            index vector or as a cell with the strings of the feature names
%            to be plotted. By default all the features are plotted.
% u:         Grid data matrix (2 x S). Each column represents a grid node
% feat:      Features data matrix (P x S). Each row is a feature. Each 
%            column can be see as the coordinates of a codebook vector
% featnames: Cell array {1 x P} with the names of the features to be
%            represented. Default: {'Feature 1','Feature 2',...}
% limits:    Two column matrix (P x 2) with the limits for each of the P variables 
%

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


% Clear figure
% clf;

% IF A SOM STRUCT IS TO BE PLOTTED
if isstruct(arg1)
    smap = arg1;
    ni        = smap.dims{1};
    nj        = smap.dims{2};
    u         = smap.pos;
    feat      = smap.w';
    featnames = smap.featnames;
    limits    = smap.limits;
    switch nargin
        case 1
            plottype = 'image';
            
        case 2
            if ischar(arg2)
                plottype = arg2;
            elseif iscellstr(arg2)
                idx = strnmatch(arg2,smap.featnames);
                plottype = 'image';
            else
                idx = arg2;
                plottype = 'image';
            end    
            
        case 3
            if iscellstr(arg2)
                idx = strnmatch(arg2,smap.featnames);
                plottype = arg3;
            else
                idx = arg2;
                plottype = arg3;
            end    
    end
    
    % A GENERAL PLANES COMMAND    
else
    u    = arg1;
    feat = arg2;
    
    switch nargin
        
        case 2
            featnames = '';
            plottype  = 'image';
            
        case 3
            if iscell(arg3)
                featnames = arg3;
                plottype = 'image';
            else
                featnames = '';
                plottype  = arg3;
            end
            
        case 4
            featnames = arg3;
            plottype  = arg4;
            
        case 5
            featnames = arg3;
            plottype  = arg4;
            limits    = arg5;
            
    end    
end

if size(u,1)>2 
    error('This function is only meant for 2D planes');
end



% Select only features specified by the user
if exist('idx')
    feat = feat(idx,:);
    featnames = featnames(idx);
end



% Determine the size (rows and columns) of the given u-lattice
[P,N] = size(feat);
if ~isstruct(arg1)
    Dx = u(1,end)-u(1,1);
    Dy = u(2,end)-u(2,1);
    dx = u(1,2)-u(1,1);
    ni = round(Dx/dx)+1;
    dy = u(2,ni+1)-u(2,1);
    nj = round(Dy/dy)+1;
end



% Assign an optimum distribution of the subplots
Ni = floor(sqrt(P));
Nj = Ni;
while Ni*Nj<P
    Nj=Nj+1;
end;

% Assign default names for variables/features
if size(featnames)==0
    for k = 1:P
        featnames{k} = ['Feature ' num2str(k)];
    end
end



if ~exist('limits')
    limits = minimaxi(feat);    
else
    if size(limits,1)==1
        limits = ones(P,1)*limits;
    end
end
    


% Draw the planes
for k = 1:P
    if limits(k,1)>=limits(k,2);
        eps = 1e-8;
        limits(k,1)=limits(k,2)-eps;
        limits(k,2)=limits(k,2)+eps;
    end
    if strncmp(plottype,'image',2)
        subplot(Ni,Nj,k);
        imagesc(u(1,:),u(2,:),reshape(feat(k,:),ni,nj)',limits(k,:));
    elseif strncmp(plottype,'surface',2)
        subplot(Ni,Nj,k);
        surf(reshape(u(1,:),ni,nj),reshape(u(2,:),ni,nj),reshape(feat(k,:),ni,nj));
        shading interp;
        if ~is_octave()
            camlight;
            lighting phong;
            camproj('p');
        end
    elseif strncmp(plottype,'contour',2)
        subplot(Ni,Nj,k);
        levels = linspace(limits(k,1),limits(k,2),20);        
        [cs,h] = contourf(reshape(u(1,:),ni,nj),reshape(u(2,:),ni,nj),reshape(feat(k,:),ni,nj),levels);
        
        
        % LINEAS DE CONTORNO POR ENCIMA DE IMAGESC (SE PUEDEN "COMENTAR" Y "DESCOMENTAR")
        imagesc(u(1,:),u(2,:),reshape(feat(k,:),ni,nj)',limits(k,:));
        hold on; [cs,h] = contour(reshape(u(1,:),ni,nj),reshape(u(2,:),ni,nj),reshape(feat(k,:),ni,nj),levels,'k'); hold off;
        % clabel(cs,h,'fontsize',8);        
        % clabel(cs,h);        
        hold off;
        
    elseif strncmp(plottype,'alternative',2)
        subplot(Ni,Nj,k);
        imagesc(u(1,:),u(2,:),reshape(feat(k,:),ni,nj)',[-1 1]);
    elseif strncmp(plottype,'a2',2)
        subplot(Ni,Nj,k);
        imagesc(u(1,:),u(2,:),reshape(feat(k,:),ni,nj)');
    end
    
    % title([num2str(k) ': ' featnames{k}],'fontsize',8);
    title([featnames{k}],'fontsize',12);
    hfig(k) = gca;

    % 2006-09-17: CODIGO ELIMINADO. EN MATLAB 7 FALLAN LOS TAMAOS EN LOS PLANOS (VER AL FINAL)
    % hbar(k) = colorbar;
    % set(hbar(k),'fontsize',8);
    
    set(gca,'YDir','normal','fontsize',8);
    grid on;
    

end


% 2006-09-17: ARREGLO PARA COMPATIBILIDAD CON MATLAB 7
pause(0.5);
for k = 1:length(hfig)
    hbar(k) = colorbar('peer',hfig(k));
    set(hbar(k),'fontsize',8);
end

