function planes3d(arg1,arg2,arg3,arg4);
% PLANES3D Draws 3D component planes
%
% CALLS
% planes3d(smap)
% planes3d(u,feat)
% planes3d(u,feat,dims)
% planes3d(u,feat,dims,featnames)
%
% Notation:
%         P = Number of features
%         S = Number of lattice points
%
% smap:      Som structure
% u:         Grid data matrix (3 x S). Each column represents a grid node
% feat:      Features data matrix (P x S). Each row is a feature. Each 
%            column can be see as the coordinates of a codebook vector
% dims:      Cell array {1 x 3} with the size of the SOM lattice in the 3 
%            dimensions of the visualization space. (if not specified, the 
%            lattice is assumed with equal number of elements in each 
%            dimension)
% featnames: Cell array {1 x P} with the names of the features to be
%            represented
%
% See also PLANES, PLOTSOM
%

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.




% Consider function call parameters
if nargin == 1
    smap = arg1;
    u = smap.pos;
    feat = smap.w';
    dims = smap.dims;
    featnames = smap.featnames;
elseif nargin == 2
    u = arg1;
    feat = arg2;
    Ni = round(size(u,2)^(1/3)); 
    dims = {Ni,Ni,Ni};
    for k = 1:size(feat,1)
        featnames{k} = ['Feature ' num2str(k)];
    end
elseif nargin == 3
    u = arg1;
    feat = arg2;
    dims = arg3;
    for k = 1:size(feat,1)
        featnames{k} = ['Feature ' num2str(k)];
    end
elseif nargin == 4
    u = arg1;
    feat = arg2;
    dims = arg3;
    featnames = arg4;
end

if size(u,1)~=3 
    error('This function is only meant for 3D planes');
end

[P,N] = size(feat);
   
% Asign an optimum distribution of the subplots
Ni = floor(sqrt(P));
Nj = Ni;
while Ni*Nj<P
  	Nj=Nj+1;
end;
ni = sqrt(N);

% Draw the planes
for k = 1:P
   subplot(Ni,Nj,k);
   PlotOnePlane3D(u,feat(k,:),dims);
   title(featnames{k});
   colorbar;
end






% Interpolate SOM to have a smooth representation 
% smap = sominterp(smap,{10,10,10})
% pos  = smap.pos;
% v    = smap.w(:,1)';
% dims = cat(2,smap.dims{:});


function PlotOnePlane3D (pos,dat,dims,limits)


dims = cat(2,dims{:});

x = reshape(pos(1,:),dims(1),dims(2),dims(3));
y = reshape(pos(2,:),dims(1),dims(2),dims(3));
z = reshape(pos(3,:),dims(1),dims(2),dims(3));
v = reshape(dat,dims(1),dims(2),dims(3));

x = permute(x,[2 1 3]);
y = permute(y,[2 1 3]);
z = permute(z,[2 1 3]);
v = permute(v,[2 1 3]);

if nargin == 3
    limits{1} = linspace(min(x(:)),max(x(:)),10);
    limits{2} = [];
    limits{3} = [];
end



slice(x,y,z,v,limits{1},limits{2},limits{3})
shading interp;
if ~is_octave()
    lighting none;
    camlight left;
    camlight right;
    cameramenu;
end
grid on;

xlabel('Dim 1');
ylabel('Dim 2');
zlabel('Dim 3');

