function psi = psirbf(x,xi,Ci,sugeno_order)
% PSIRBF Regularized gaussian radial basis function
%
%        psi = psirbf(x,xi,Ci,sugeno_order)
%
% INPUTS
%           x: Points in the joint space by columns (RxQ)
%          xi: centers of the gaussians (RxNc)
%          Ci: N-Array of covariance matrix (RxRxNc)
%sugeno_order: if 0 a column of 1's is added to consider a bias term in RBF interpolation
%              if 1 a R+1 columns with [1, x1, x2, ..., xR] are added to the normalized
%              computation of exp(). This allows for Sugeno Radial Basis
%              Functions
%
% DESCRIPTION
% Obtains in a matrix the values of a regularized multivariate exp()
% function. Regularization takes place by
%
%    psi_j = exp(xj)/{exp(x1)+exp(x2)+...+exp(xn)}
%
% With sugeno_order parameter, also includes the possibility to add linear
% and bias terms [1, x1, ..., xR] which are also normalized.
%
% See also MVG

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



phi = exprbf(x,xi,Ci); 
R = size(x,1);
N = size(x,2);
S = size(xi,2);

psi = phi./(sum(phi,2)*ones(1,S));

if nargin==4
    if length(sugeno_order) == 0
        return;
    elseif sugeno_order == 0
        psi = [psi ones(N,1)];
    elseif sugeno_order == 1
%         idx  = rectop([1:R+1],[1:S+1],'idx');
%         psi1 = [psi 0*ones(N,1)];
         idx  = rectop([1:R+1],[1:S],'idx');
         psi1 = [psi];
        x1    = [x'  ones(N,1)];
        psi   = psi1(:,idx(2,:)).*x1(:,idx(1,:));
    end
end

