function [mf,rule] = fuzzymaps(som);

% FUZZYMAPS Obtains SOM fuzzy maps interactively
%
% CALLS:
% [mf,rule] = fuzzymaps(som)
%
%
% DESCRIPTION:
% First of all, a figure will appear with colorbar images with the same range as the
% data corresponding to each SOM plane.
%
% The user should click twice the bars to determine ranges.
%     Left  button = AND operator with respect to previous items
%     Right button = OR  operator with respect to previous items
% Operators 'AND' and 'OR' follow the same precedence as '*' and '+'
%
% Example:
% feature1 in (1.3,1.5) AND     (2 left clicks in plane 1)
% feature3 in (1.2,4.5) AND     (2 left clicks in plane 3)
% feature2 in (1.1,3.5)         (2 left clicks in plane 2)
% OR
% feature5 in (0.3,2.5) AND     (2 right clicks in plane 5)
% feature2 in (1.1,2.5) AND     (2 left clicks in plane 2)
% feature6 in (2.2,3.5)         (2 left clicks in plane 6)
%
% 
% In a second figure will appear the planes with white dots marking the regions which
% fulfill the conditions with above an 85% of truthness
% A third figure will show up the fuzzy map with membership values.
%
% OUTPUTS:
%   mf: is a (1xS) vector with the fuzzy plane, ready to be used 
%       with e.g.: 
%                planes(som.pos,mf)
% rule: is string with the selected rule.
%

%   Copyright (c) 2007, by 
%   Grupo de Supervisin y Diagnstico de Procesos Industriales
%   Area de Ingeniera de Sistemas y Automtica
%   Universidad de Oviedo
%   Contributed files may contain copyrights of their own.
%   
%   UniOviSOM toolbox is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


N = length(som.featnames);
w = som.w';
featnames = som.featnames;
mnmx=minimaxi(w);
minw=mnmx(:,1);
maxw=mnmx(:,2);

figure(1); clf;
% Asign an optimum distribution of the subplots
Ni = floor(sqrt(N));
Nj = Ni;
while Ni*Nj<N
  	Nj=Nj+1;
end;

figure(1);
for k = 1:N, 
    subplot(Ni,Nj,k); 
    imagesc(zeros(128,1),linspace(minw(k),maxw(k),128)',linspace(minw(k),maxw(k),128)'); 
    set(gca,'plotboxaspectratio',[0.2 1 1]);
    set(gca,'xtick',[]);
    set(gca,'fontsize',5);
    set(gca,'ydir','normal');
    title(featnames(k),'fontsize',7);
end;


rule = {''};
mf(1,:) = ones(size(w(1,:)));
hc = get(gcf,'children');
k = 1;
while 1   
    figure(1);
    [x,y,b]=ginput(2);
    if length(b)==0
        break;
    end
    if b(2)==3
        k = k+1;
        mf(k,:) = ones(size(w(1,:)));
        rule{k} = 'OR ';
    end
    idx   = N-find(hc==gca)+1;
    range = maxw(idx)-minw(idx);
    grnn  = grnntra([0 1 1 0],[min(y)-range/5 min(y) max(y) max(y)+range/5],range/10);
    t = linspace(minw(idx),maxw(idx),100);
    % plot(t,grnnsim(t,grnn));
    mf(k,:) = min([mf(k,:);grnnsim(w(idx,:),grnn)]);
    rule{k} = strvcat(rule{k},[featnames{idx} ' IN (' num2str(min(y)) ',' num2str(max(y)) ')']);
    strvcat(rule),
end

mf   = max(mf,[],1);
rule = strvcat(rule);

figure(2);
planes(som.pos,mf,'alt');
text(som.dims{1}/2,som.dims{2}/2,rule,'fontsize',8,'hor','cen','ver','mid');
figure(3);
idx = find(mf>0.9);
planes(som); plotp(som.pos(:,idx),'ko',1.5,'all');


